/*
 * PSP Software Development Kit - http://www.pspdev.org
 * -----------------------------------------------------------------------
 * Licensed under the BSD license, see LICENSE in PSPSDK root for details.
 * ELF Based WLAN Scanner
 * Copyright (c) 2005 James F <tyranid@gmail.com>
 */
// correction pour tenir compte du mode graphique
#include <pspkernel.h>
#include <pspdebug.h>
#include <pspsdk.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <pspnet.h>
#include <pspwlan.h>
#include <pspctrl.h>
#include <pspdisplay.h>
#include <psppower.h>
#include <math.h>
#include <png.h>
#include "graphics.h"

#define printf pspDebugScreenPrintf

#define MODULE_NAME "WlanScanner"

PSP_MODULE_INFO(MODULE_NAME, 0x1000, 1, 1);
PSP_MAIN_THREAD_ATTR(0);

/* Init the scan */
int sceNet_lib_5216CBF5(const char *name);
/* Do the scan */
int sceNet_lib_7BA3ED91(const char *name, void *type, u32 *size, void *buf, u32 *unk);
/* Terminate the scan */
int sceNet_lib_D2422E4D(const char *name);

#define InitScan sceNet_lib_5216CBF5
#define ScanAPs  sceNet_lib_7BA3ED91
#define TermScan sceNet_lib_D2422E4D

/* Global buffer to store the scan data */
unsigned char scan_data[0xA80];

char sauv=0,txt=1,global_count=0;
SceUID f;

/* Returned data */
struct ScanData
{
        struct ScanHead *pNext; 
        unsigned char bssid[6]; 
        char channel; 
        unsigned char namesize; 
        char name[32]; 
        unsigned int bsstype; 
        unsigned int beaconperiod; 
        unsigned int dtimperiod; 
        unsigned int timestamp; 
        unsigned int localtime; 
        unsigned short atim; 
        unsigned short capabilities; 
        unsigned char  rate[8]; 
        unsigned short rssi; 
        unsigned char  sizepad[6]; 
} __attribute__((packed));

/* Capability flags */
const char *caps[8] = {
        "ESS, ",
        "IBSS, ",
        "CF Pollable, ",
        "CF Pollreq,  ",
        "Privacy (WEP), ",
        "Short Preamble, ",
        "PBCC, ",
        "Channel Agility, "
};

void nextname(char *base)
{char s[52];
 int fcpt=0;
 SceUID f;
 do {sprintf(s,"ms0:/%s%d.dat",base,fcpt);
     f=sceIoOpen(s,PSP_O_RDONLY,0777);
     if (f>0) {sceIoClose (f);fcpt++;} 		// f>=3 if file exists
 } while (f>0);
 sprintf(base,"%s",s);
}

/* Print the scan summary data to stdout */
void print_apsum(struct ScanData *pData)
{
        char name[33];
        char bssid[30];
	char s[80];

        strncpy(name, pData->name, 32);
        name[32] = 0;
        sceNetEtherNtostr(pData->bssid, bssid);
        if (txt==1) {printf("%03d  ", pData->rssi);
                     if (pData->capabilities & (1 << 4))
                         printf(" WEP! ");
                     else
                         printf("NO WEP");
                     
                     printf("\t%s", bssid);
                     printf("\t%s\n", name);
	}     
	if (sauv==1)
	   {sprintf(s,"%03d %s %s\n",pData->rssi,bssid,name);
            sceIoWrite(f,s,strlen(s)); 		// save in File
	   }
}

/* Print the scan data to stdout */
void print_ap(struct ScanData *pData)
{
        char name[33];
        char bssid[30];
        int  loop;

        strncpy(name, pData->name, 32);
        name[32] = 0;
        sceNetEtherNtostr(pData->bssid, bssid);
        printf("BSSID: %s\n", bssid);
        printf("SSID: %s\n", name);
        printf("bsstype: ");
        if(pData->bsstype == 1)
        {printf("Infrastructure\n");}
        else if(pData->bsstype == 2)
          {printf("Independent\n");}
             else {printf("Unknown\n");}
        printf("Beacon Period: %d\n", pData->beaconperiod);
        printf("DTIM period: %d\n", pData->dtimperiod);
        printf("Timestamp: %d\n", pData->timestamp);
        printf("Local Time: %d\n", pData->localtime);
        printf("Channel: %d\n", pData->channel);
        printf("ATIM: %d\n", pData->atim);
        printf("Capability Information: ");
        for(loop = 0; loop < 8; loop++)
        {
                if(pData->capabilities & (1 << loop))
                {
                        printf("%s", caps[loop]);
                }
        }
        printf("\n");

        printf("Rate: ");
        for(loop = 0; loop < 8; loop++)
        {
                const char *type;

                if(pData->rate[loop] & 0x80) {type = "Basic";}
                else {type = "Operational";}

                if(pData->rate[loop] & 0x7F)
                {printf("%s %d kbps, ", type, (pData->rate[loop] & 0x7F)*500);}
        }
        printf("\n");
        printf("RSSI: %d\n", pData->rssi);
        printf("\n");
}

/* Do a scan */
struct ScanData *do_scan(int *count)
{
        unsigned char type[0x4C];
        u32 size, unk;
        int i;
        int ret;

        if((ret = InitScan("wlan")) >= 0)
        {
                memset(type, 0, sizeof(type));
                /* Set the channels we want to scan */
                for(i = 1; i < 0xF; i++) {type[0x9+i] = i;}
                type[0x3C] = 1;
                *((u32*) (type + 0x44)) = 1;    //6/* Minimum strength */
                *((u32*) (type + 0x48)) = 100;  /* Maximum strength */
                size = sizeof(scan_data);
                unk  = 0;
                memset(scan_data, 0, sizeof(scan_data));
                ret = ScanAPs("wlan", type, &size, scan_data, &unk);
                if(ret < 0)
                {printf("Error, could not perform scan err = %08X\n", ret);}
                else
                {*count = size / sizeof(struct ScanData);
                 return (struct ScanData *) scan_data;
                }
        }
        else
        {
                printf("Error, cannot initialise scan %08X\n", ret);
        }
        TermScan("wlan");
        return NULL;
}

/* Exit callback */
int exit_callback(int arg1, int arg2, void *common)
{
        sceWlanDevDetach();
        sceNetTerm();
        sceKernelExitGame();
        return 0;
}

/* Callback thread */
int CallbackThread(SceSize args, void *argp)
{
        int cbid;

        cbid = sceKernelCreateCallback("Exit Callback", exit_callback, NULL);
        sceKernelRegisterExitCallback(cbid);
        sceKernelSleepThreadCB();

        return 0;
}

/* Sets up the callback thread and returns its thread id */
int SetupCallbacks(void)
{
        int thid = 0;

        thid = sceKernelCreateThread("update_thread", CallbackThread,
                                     0x11, 0xFA0, PSP_THREAD_ATTR_USER, 0);
        if(thid >= 0) {sceKernelStartThread(thid, 0, 0);}
        return thid;
}

void start_ui(struct ScanData *pScan, int count)
{int x;

 sceCtrlSetSamplingCycle(0);
 sceCtrlSetSamplingMode(PSP_CTRL_MODE_ANALOG);
 if (txt==1) printf("Scan found %d access point(s)\n\n", count);

 for (x=0;x<count;x++) {print_apsum(&pScan[x]);}
//                        print_ap(&pScan[ap-1]);
 global_count=count;
}

int net_thread() // SceSize args, void *argp)
{
        struct ScanData *pScan = NULL;
        int count;
        int ret;
 
        ret = sceNetInit(0x20000, 0x20, 0x1000, 0x20, 0x1000);
        if(ret < 0)
          {printf("Error initialising network lib %08X\n", ret);
           goto error;
          }

        ret = -1;
        while(ret < 0) {ret = sceWlanDevAttach();

         /* If returns a error which indicates it isn't ready */
                        if(ret == 0x80410D0E)
                          {sceKernelDelayThread(1000000);}
                           else if(ret < 0)
                               {printf("Error attaching to wlan device %08X\n", ret);
                                goto error;
                               }
                          }
        pScan = do_scan(&count);
        start_ui(pScan, count);
error:
        sceWlanDevDetach();
        sceNetTerm();

        return 0;
}

/* Simple thread */
int main(int argc, char **argv)
 {SceCtrlData pad;
  int bauds[7]={1200,2400,4800,9600,19200,38400,57600},cpt=2;
  SceUID scr;
  char s[80];
  int ch,fcpt=0,flag=0,ilat=0,ilon=0,i;
  double flat=0.0,flon=0.0,oflat=0.,oflon=0.,mag=128.,degla,deglo;
  char lat[16],lon[16],nmea=0,fini=0;

  SetupCallbacks();
  pspDebugScreenInit();
//RS232
  pspDebugScreenSetTextColor (0xffffffff);  // White
  pspDebugScreenPrintf("[] and O to change baud rate, ^ to toggle save\n");
  pspDebugScreenPrintf("UP to toggle txt/graphic mode, X to validate and quit");
  pspDebugScreenSetXY(1,6);
  pspDebugScreenPrintf("Select: clear screen\n");
  pspDebugScreenPrintf("Start: dump screen\n");
  pspDebugSioInit();                        // init RS232

  while (1) {
     pspDebugScreenSetXY(1,3);
     pspDebugScreenPrintf("baud=%d  \tsave=%d \ttxt=%d ",bauds[cpt],sauv,txt);
     sceCtrlPeekBufferPositive(&pad, 1); 
     if (pad.Buttons & PSP_CTRL_SQUARE) {cpt++;if (cpt>6) cpt=0;}
     if (pad.Buttons & PSP_CTRL_CIRCLE) {cpt--;if (cpt<0) cpt=6;}
     if (pad.Buttons & PSP_CTRL_TRIANGLE) {sauv=1-sauv;}
     if (pad.Buttons & PSP_CTRL_UP) {txt=1-txt;}
     if (pad.Buttons & PSP_CTRL_CROSS) {break;}
     sceKernelDelayThread(200000); // delay 200 ms = debounce
  }
  printf("\n\n\n\n");
// let's start
  if (sauv==1) {sprintf(s,"gps"); nextname(s);	// returns in s the new filename
                f=sceIoOpen(s,PSP_O_WRONLY|PSP_O_CREAT|PSP_O_APPEND,0777);}

  pspDebugSioSetBaud(bauds[cpt]);
  printf("\nPSP GPS JMF test");
  if (sauv==1) pspDebugScreenPrintf(": saving in %s\n",s); else
               pspDebugScreenPrintf("\n");
  sceKernelDelayThread(1000000); 	// delay 1 s

  if (txt==0) {
    InitializeGraphics(1);		//  2 bytes/pixel && 512 instead of 480
    // initGraphics();
    memset((char*)0x44000000,0,512*272*2);  			// blank screen 
    }
// end of RS232 init
        if(pspSdkLoadInetModules() < 0)
        {printf("Error, could not load inet modules\n");
         sceKernelSleepThread();
        }

do
{ cpt=0;fcpt=0;
  nmea=0;
  do {
           ch = pspDebugSioGetchar();	// get new RS232 char
           if ((ch >= 0) && (ch != '\r')) {
//	        if (sauv==1) {sceIoWrite(f,&ch,1);}		// save in File
//              if (txt==1) 
//                pspDebugScreenPrintf("%c", ch);	// print
                if (((ch=='G')||(ch=='P')||(ch=='A')||(flag>99))) flag++; 
                                                                  else flag=0;
                if (flag==5) {flag=100;fcpt=0;}                 // on a eu GPGGA
                if ((flag>112) && (flag<122)) {lat[flag-113]=ch;}
                if ((flag>124) && (flag<135)) {lon[flag-125]=ch;}
                if (flag==135) {                     	// fini parsing $GPGGA
                        lat[9]=0;lon[10]=0;		// end of string
                        flat=atof(lat);flon=atof(lon);
			degla=floor(flat/100.)*100.; 
			deglo=floor(flon/100.)*100.;
			flat=degla+(flat-degla)/60.*100.; // 0..60 -> 0..100 lat
			flon=deglo+(flon-deglo)/60.*100.; // 0..60 -> 0..100 lon
                        if ((oflat==0.) && (oflon==0.))
                             {oflat=flat;oflon=flon;}   // init pos
			ilat=(int)(floor((flat-oflat)*mag))+136;
			ilon=(int)(floor((flon-oflon)*mag))+240;
                        flag=0;nmea=1;}
           }
           sceCtrlPeekBufferPositive(&pad, 1);  	// DON'T wait VSync
           if (pad.Buttons & PSP_CTRL_CROSS) {fini=1;}
           if (fcpt==0) {
               if (pad.Buttons & PSP_CTRL_LEFT)   {oflon-=10.;fcpt=1;}
               if (pad.Buttons & PSP_CTRL_RIGHT)  {oflon+=10.;fcpt=1;}
               if (pad.Buttons & PSP_CTRL_UP)     {oflat+=10.;fcpt=1;}
               if (pad.Buttons & PSP_CTRL_DOWN)   {oflat-=10.;fcpt=1;}
               if (pad.Buttons & PSP_CTRL_SQUARE) {mag*=2.;fcpt=1;}
               if (pad.Buttons & PSP_CTRL_CIRCLE) {if (mag>1) mag/=2.;fcpt=1;}
               if (pad.Buttons & PSP_CTRL_SELECT) {
                   memset((char*)0x44000000,0,512*272*2);	// blank screen
                   oflat=0.;oflon=0.;}
               if (pad.Buttons & PSP_CTRL_START) {
                   sprintf(s,"dump"); nextname(s);  // returns the new filename
                   scr=sceIoOpen(s,PSP_O_WRONLY|PSP_O_CREAT|PSP_O_APPEND,0777);
                   sceIoWrite(scr,lat,9);	// dump screen	
                   sceIoWrite(scr,lon,10);
                   for (i=0;i<272;i++) {
                      sceIoWrite(scr,(char*)(0x44000000+2*i*512),480*2); // line
                      PlotPixel(1,i,155,155,155);}
                      // putPixelScreen(0xff808080,1,i);}
                   sceIoClose(scr);
                   // screenshot("ms0:/d.png");
               }
       }

  } while (nmea==0);
 
  if (txt==1) 
     {pspDebugScreenClear();
      printf("-> %s=%lf/%s=%lf mag=%lf\n",lat,flat,lon,flon,mag);}
  if (sauv==1) {
	  sprintf(s," %s=%lf %s=%lf \n",lat,flat,lon,flon);     // save in File
          sceIoWrite(f,s,strlen(s));         // synchronise donnees sur le MS ?
          // sceIoSync("ms0:",1);
	  }		
          

  net_thread();
  if (txt==0) if ((ilat>0) && (ilat<272) && (ilon>0) && (ilon<480))
         PlotPixel(ilon,271-ilat,255-8*global_count,255,255); // max 32 AP can be detected
         // putPixelScreen(0xffffffff,ilon,271-ilat);
 } while (fini==0);

 if (sauv==1) sceIoClose(f);
 if (txt==1) pspDebugScreenPrintf("Good bye");
 // sceKernelExitDeleteThread(0);
 sceKernelExitGame();

 return 0;
}
